@@ -17,32 +17,32 @@ var (
 
 // uapi/linux/capability.h defined.
 const (
-	VFS_CAP_REVISION_MASK   = uint32(0xff000000)
-	VFS_CAP_FLAGS_MASK      = ^VFS_CAP_REVISION_MASK
-	VFS_CAP_FLAGS_EFFECTIVE = uint32(1)
+	vfsCapRevisionMask   = uint32(0xff000000)
+	vfsCapFlagsMask      = ^vfsCapRevisionMask
+	vfsCapFlagsEffective = uint32(1)
 
-	VFS_CAP_REVISION_1 = uint32(0x01000000)
-	VFS_CAP_REVISION_2 = uint32(0x02000000)
-	VFS_CAP_REVISION_3 = uint32(0x03000000)
+	vfsCapRevision1 = uint32(0x01000000)
+	vfsCapRevision2 = uint32(0x02000000)
+	vfsCapRevision3 = uint32(0x03000000)
 )
 
 // Data types stored in little-endian order.
 
-type vfs_caps_1 struct {
+type vfsCaps1 struct {
 	MagicEtc uint32
 	Data     [1]struct {
 		Permitted, Inheritable uint32
 	}
 }
 
-type vfs_caps_2 struct {
+type vfsCaps2 struct {
 	MagicEtc uint32
 	Data     [2]struct {
 		Permitted, Inheritable uint32
 	}
 }
 
-type vfs_caps_3 struct {
+type vfsCaps3 struct {
 	MagicEtc uint32
 	Data     [2]struct {
 		Permitted, Inheritable uint32
@@ -52,11 +52,18 @@ type vfs_caps_3 struct {
 
 // ErrBadSize indicates the the loaded file capability has
 // an invalid number of bytes in it.
-var (
-	ErrBadSize  = errors.New("filecap bad size")
-	ErrBadMagic = errors.New("unsupported magic")
-	ErrBadPath  = errors.New("file is not a regular executable")
-)
+var ErrBadSize = errors.New("filecap bad size")
+
+// ErrBadMagic indicates that the kernel preferred magic number for
+// capability Set values is not supported by this package. This
+// generally implies you are using an exceptionally old
+// "../libcap/cap" package. An upgrade is needed, or failing that see
+// https://sites.google.com/site/fullycapable/ for how to file a bug.
+var ErrBadMagic = errors.New("unsupported magic")
+
+// ErrBadPath indicates a failed attempt to set a file capability on
+// an irregular (non-executable) file.
+var ErrBadPath = errors.New("file is not a regular executable")
 
 // digestFileCap unpacks a file capability and returns it in a *Set
 // form.
@@ -64,9 +71,9 @@ func digestFileCap(d []byte, sz int, err error) (*Set, error) {
 	if err != nil {
 		return nil, err
 	}
-	var raw1 vfs_caps_1
-	var raw2 vfs_caps_2
-	var raw3 vfs_caps_3
+	var raw1 vfsCaps1
+	var raw2 vfsCaps2
+	var raw3 vfsCaps3
 	if sz < binary.Size(raw1) || sz > binary.Size(raw3) {
 		return nil, ErrBadSize
 	}
@@ -78,36 +85,36 @@ func digestFileCap(d []byte, sz int, err error) (*Set, error) {
 
 	c := NewSet()
 	b.Seek(0, io.SeekStart)
-	switch magicEtc & VFS_CAP_REVISION_MASK {
-	case VFS_CAP_REVISION_1:
+	switch magicEtc & vfsCapRevisionMask {
+	case vfsCapRevision1:
 		if err = binary.Read(b, binary.LittleEndian, &raw1); err != nil {
 			return nil, err
 		}
 		data := raw1.Data[0]
 		c.flat[0][Permitted] = data.Permitted
 		c.flat[0][Inheritable] = data.Inheritable
-		if raw1.MagicEtc&VFS_CAP_FLAGS_MASK == VFS_CAP_FLAGS_EFFECTIVE {
+		if raw1.MagicEtc&vfsCapFlagsMask == vfsCapFlagsEffective {
 			c.flat[0][Effective] = data.Inheritable | data.Permitted
 		}
-	case VFS_CAP_REVISION_2:
+	case vfsCapRevision2:
 		if err = binary.Read(b, binary.LittleEndian, &raw2); err != nil {
 			return nil, err
 		}
 		for i, data := range raw2.Data {
 			c.flat[i][Permitted] = data.Permitted
 			c.flat[i][Inheritable] = data.Inheritable
-			if raw2.MagicEtc&VFS_CAP_FLAGS_MASK == VFS_CAP_FLAGS_EFFECTIVE {
+			if raw2.MagicEtc&vfsCapFlagsMask == vfsCapFlagsEffective {
 				c.flat[i][Effective] = data.Inheritable | data.Permitted
 			}
 		}
-	case VFS_CAP_REVISION_3:
+	case vfsCapRevision3:
 		if err = binary.Read(b, binary.LittleEndian, &raw3); err != nil {
 			return nil, err
 		}
 		for i, data := range raw3.Data {
 			c.flat[i][Permitted] = data.Permitted
 			c.flat[i][Inheritable] = data.Inheritable
-			if raw3.MagicEtc&VFS_CAP_FLAGS_MASK == VFS_CAP_FLAGS_EFFECTIVE {
+			if raw3.MagicEtc&vfsCapFlagsMask == vfsCapFlagsEffective {
 				c.flat[i][Effective] = data.Inheritable | data.Permitted
 			}
 		}
@@ -118,11 +125,13 @@ func digestFileCap(d []byte, sz int, err error) (*Set, error) {
 	return c, nil
 }
 
+//go:uintptrescapes
+
 // GetFd returns the file capabilities of an open (*os.File).Fd().
 func GetFd(file *os.File) (*Set, error) {
-	var raw3 vfs_caps_3
+	var raw3 vfsCaps3
 	d := make([]byte, binary.Size(raw3))
-	sz, _, oErr := callRKernel6(syscall.SYS_FGETXATTR, uintptr(file.Fd()), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0)
+	sz, _, oErr := multisc.r6(syscall.SYS_FGETXATTR, uintptr(file.Fd()), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0)
 	var err error
 	if oErr != 0 {
 		err = oErr
@@ -130,15 +139,17 @@ func GetFd(file *os.File) (*Set, error) {
 	return digestFileCap(d, int(sz), err)
 }
 
+//go:uintptrescapes
+
 // GetFile returns the file capabilities of a named file.
 func GetFile(path string) (*Set, error) {
 	p, err := syscall.BytePtrFromString(path)
 	if err != nil {
 		return nil, err
 	}
-	var raw3 vfs_caps_3
+	var raw3 vfsCaps3
 	d := make([]byte, binary.Size(raw3))
-	sz, _, oErr := callRKernel6(syscall.SYS_GETXATTR, uintptr(unsafe.Pointer(p)), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0)
+	sz, _, oErr := multisc.r6(syscall.SYS_GETXATTR, uintptr(unsafe.Pointer(p)), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0)
 	if oErr != 0 {
 		err = oErr
 	}
@@ -153,6 +164,21 @@ func (c *Set) GetNSOwner() (int, error) {
 	return c.nsRoot, nil
 }
 
+// SetNSOwner adds an explicit namespace owner UID to the capability
+// Set. This is only honored when generating file capabilities, and is
+// generally for use by a setup process when installing binaries that
+// use file capabilities to become capable inside a namespace to be
+// administered by that UID. If capability aware code within that
+// namespace writes file capabilities without explicitly setting such
+// a UID, the kernel will fixup the capabilities to be specific to
+// that owner. In this way, the kernel prevents filesystem
+// capabilities from leaking out of that restricted namespace.
+func (c *Set) SetNSOwner(uid int) {
+	c.mu.Lock()
+	defer c.mu.Unlock()
+	c.nsRoot = uid
+}
+
 // packFileCap transforms a system capability into a VFS form. Because
 // of the way Linux stores capabilities in the file extended
 // attributes, the process is a little lossy with respect to effective
@@ -164,13 +190,13 @@ func (c *Set) packFileCap() ([]byte, error) {
 		if c.nsRoot != 0 {
 			return nil, ErrBadSet // nsRoot not supported for single DWORD caps.
 		}
-		magic = VFS_CAP_REVISION_1
+		magic = vfsCapRevision1
 	case 2:
 		if c.nsRoot == 0 {
-			magic = VFS_CAP_REVISION_2
+			magic = vfsCapRevision2
 			break
 		}
-		magic = VFS_CAP_REVISION_3
+		magic = vfsCapRevision3
 	}
 	if magic == 0 {
 		return nil, ErrBadSize
@@ -180,7 +206,7 @@ func (c *Set) packFileCap() ([]byte, error) {
 		eff |= (f[Permitted] | f[Inheritable]) & f[Effective]
 	}
 	if eff != 0 {
-		magic |= VFS_CAP_FLAGS_EFFECTIVE
+		magic |= vfsCapFlagsEffective
 	}
 	b := new(bytes.Buffer)
 	binary.Write(b, binary.LittleEndian, magic)
@@ -194,23 +220,32 @@ func (c *Set) packFileCap() ([]byte, error) {
 	return b.Bytes(), nil
 }
 
+//go:uintptrescapes
+
 // SetFd attempts to set the file capabilities of an open
-// (*os.File).Fd(). Note, Linux does not store the full Effective
-// Value vector in the metadata for the file. Only a single Effective
-// bit is stored. This single bit is non-zero if the Permitted vector
-// have any overlapping bits with the Effective or Inheritable vector
-// of c. This may seem suboptimal, but the reasoning behind it is
-// sound. Namely, the purpose of the Effective bit it to support
-// capabability unaware binaries that can work if they magically
-// launch with only the needed bits raised (this bit is sometimes
-// referred to simply as the 'legacy' bit). However, the preferred way
-// a binary will actually manipulate its file-acquired capabilities is
-// carefully and deliberately using this package, or libcap for C
-// family code. This function can also be used to delete a file's
+// (*os.File).Fd(). This function can also be used to delete a file's
 // capabilities, by calling with c = nil.
+//
+// Note, Linux does not store the full Effective Value Flag in the
+// metadata for the file. Only a single Effective bit is stored in
+// this metadata. This single bit is non-zero if the Permitted vector
+// has any overlapping bits with the Effective or Inheritable vector
+// of c. This may appear suboptimal, but the reasoning behind it is
+// sound. Namely, the purpose of the Effective bit it to support
+// capabability unaware binaries that will only work if they magically
+// launch with the needed bits already raised (this bit is sometimes
+// referred to simply as the 'legacy' bit). Without *full* support for
+// capability manipulation, as it is provided in this "../libcap/cap"
+// package, this was the only way for Go programs to make use of
+// capabilities.
+//
+// The preferred way a binary will actually manipulate its
+// file-acquired capabilities is to carefully and deliberately using
+// this package (or libcap, assisted by libpsx, for threaded C/C++
+// family code).
 func (c *Set) SetFd(file *os.File) error {
 	if c == nil {
-		if _, _, err := callRKernel6(syscall.SYS_FREMOVEXATTR, uintptr(file.Fd()), uintptr(unsafe.Pointer(xattrNameCaps)), 0, 0, 0, 0); err != 0 {
+		if _, _, err := multisc.r6(syscall.SYS_FREMOVEXATTR, uintptr(file.Fd()), uintptr(unsafe.Pointer(xattrNameCaps)), 0, 0, 0, 0); err != 0 {
 			return err
 		}
 		return nil
@@ -221,17 +256,20 @@ func (c *Set) SetFd(file *os.File) error {
 	if err != nil {
 		return err
 	}
-	if _, _, err := callRKernel6(syscall.SYS_FSETXATTR, uintptr(file.Fd()), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0); err != 0 {
+	if _, _, err := multisc.r6(syscall.SYS_FSETXATTR, uintptr(file.Fd()), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0); err != 0 {
 		return err
 	}
 	return nil
 }
 
+//go:uintptrescapes
+
 // SetFile attempts to set the file capabilities of the specfied
-// filename. See the comment for SetFd() for some non-obvious behavior
-// of Linux for the Effective Value vector on the modified file.  This
-// function can also be used to delete a file's capabilities, by
-// calling with c = nil.
+// filename. This function can also be used to delete a file's
+// capabilities, by calling with c = nil.
+//
+// Note, see the comment for SetFd() for some non-obvious behavior of
+// Linux for the Effective Value vector on the modified file.
 func (c *Set) SetFile(path string) error {
 	fi, err := os.Stat(path)
 	if err != nil {
@@ -249,7 +287,7 @@ func (c *Set) SetFile(path string) error {
 		return err
 	}
 	if c == nil {
-		if _, _, err := callRKernel6(syscall.SYS_REMOVEXATTR, uintptr(unsafe.Pointer(p)), uintptr(unsafe.Pointer(xattrNameCaps)), 0, 0, 0, 0); err != 0 {
+		if _, _, err := multisc.r6(syscall.SYS_REMOVEXATTR, uintptr(unsafe.Pointer(p)), uintptr(unsafe.Pointer(xattrNameCaps)), 0, 0, 0, 0); err != 0 {
 			return err
 		}
 		return nil
@@ -260,7 +298,7 @@ func (c *Set) SetFile(path string) error {
 	if err != nil {
 		return err
 	}
-	if _, _, err := callRKernel6(syscall.SYS_SETXATTR, uintptr(unsafe.Pointer(p)), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0); err != 0 {
+	if _, _, err := multisc.r6(syscall.SYS_SETXATTR, uintptr(unsafe.Pointer(p)), uintptr(unsafe.Pointer(xattrNameCaps)), uintptr(unsafe.Pointer(&d[0])), uintptr(len(d)), 0, 0); err != 0 {
 		return err
 	}
 	return nil
